import datetime
import time
from data_mod import DataMod
import sys
import os
from glob import glob
import sets
from dateutil.relativedelta import *
import cdo as cdop
import copy
from scipy.io.netcdf import *
import numpy
import shutil
from common import *
#from nco import Nco


class DataEC3(DataMod):
    """ classe of data for all experiments realized 
    with autosubmit with the standard outclass 
    """
    def __init__(self, dataname, varname):
        DataMod.__init__(self, dataname, varname)
        self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 6))

    def set_inputdirAuto(self):
        """defining the path to look for the model daily data"""
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        inputdir = os.path.join("/esnas/exp/ecearth/",
                                dataname,"/original_files/")
        # test for data organized according to the new/old data structure
        if (not os.path.isdir(inputdir)):
            inputdir = os.path.join("/esnas/exp/ecearth/rawfiles/",dataname)
            if (not os.path.isdir(inputdir)):
                print "This directory does not exist:"
                print inputdir
                sys.exit(0)
        self.set_inputdir(inputdir)

    def set_outputdirAuto(self):
        """defining the path to store the output"""
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        outputdir = "/esnas/exp/ecearth/"+dataname+"/"+""
        if (not os.path.isdir(outputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)

    def set_files(self, date1, date2, memblst=[]):
        """set the list of stardates and correponding directories
        as for these type of experiments the members and leadtime
        are stored in different file we first find the correct 
        path to the directories"""

        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]
        filetype="dir"
        # Check if the files are already in the working directory
        if (inputdir=="./"):
            filetype="nc"
            filebase=self.get_varname() + "_" + date1.strftime("%Y%m%d")+".nc"
            filename=glob(inputdir+filebase)
            if len(filename)== 0:
                print("no nc file in "+inputdir)
                return 1
            filelist.append(filename[0])
            datelist.append(date1)
            date1 = date1 + relativedelta(years=+1)
            d=date1
            #loop on the date to find all the files
            while d <= date2:
                filebase=self.get_varname() + "_" +d.strftime("%Y%m%d") +".nc"
                filename=glob(os.path.join(inputdir,filebase))
                if len(filename)== 0:
                    print "the file %s does not exit " % inputdir+filebase+"."+filetype
                    print "while the file %s exists "  % inputdir+filelist[0]
                    print "all file must have the same type (grb or nc)"
                    return 1
                datelist.append(d)
                filelist.append(filename[0])
                d = d + relativedelta(years=+1)
        else:
            if (not os.path.isdir(inputdir)):
                print "This directory does not exist:"
                print inputdir
                return 1

            filebase=os.path.join(inputdir,
                                  date1.strftime("%Y%m%d"),"fc0","outputs")
            filelist.append(filebase)
            datelist.append(date1)
            d=date1 + relativedelta(years=+1)
            #loop on the date to find all the files
            while d <= date2:
                filebase=os.path.join(inputdir,
                                  d.strftime("%Y%m%d"),"fc0","outputs")
                datelist.append(d)
                filelist.append(filebase)
                d = d + relativedelta(years=+1)

        self.set_filetype(filetype)
        self.set_filelist(filelist)
        self.set_sdates(datelist)
     
    def format_files_nc(self,memblst=[], foretimelst=[]):
        """transform the grb file 
        into ncdf file readable for cdo
        memblst: list of member to postprocess (list of integers)
                 if empty take of the member available
        foretimelst: list of forecast time to post-process
        """
         

        #needed to use cdo
        cdo=cdop.Cdo()
        #make sure that the tmp dir is not /tmp
        cdo.tempfile.tempdir = os.environ['TMPDIR']
        
        #get the needed variables from the object
        inputdir=self.get_inputdir()
        dirlist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        nmember=self.get_nmember()
        filetype=self.get_filetype()
        filelist=[]
             

        #define name of var in the file
        dictvar={"tas":"T2M","lsp":"LSP","cp":"CP","psl":"MSL","tos":"SSTK",
                 "ta":"T", "ua":"U", "va":"V", "hfls":"SLHF",
                 "hfss":"SSHF", "rss":"SSR", "rls":"STRD"
        }
        oldname=dictvar.get(varname)
        if oldname == None:
            print("the variable "+varname+"is no available in the format_file function")
            sys.exit(1)
        #the files are stored in on directory
        #this function works only for these configuration
        #sdates/menbers/file per leadtime
        if (filetype != "dir"):
            print("the function is done for directory type of file")
            print("not "+filetype+"type of file")
            return 1
        freq=self.get_freq()
        #get the number of members, which can be define as an argument
        #otherwise all the members are used
        nmember=len(memblst)
        if len(memblst)==0:
            lstdir=glob(os.path.join(inputdir,
                                     (sdates[0].strftime("%Y%m%d")+"/*")))
            lstdir.sort()
            memblst=[int(x.split("/fc")[-1]) for x in lstdir]
            nmember=len(memblst)
          
        #loop over the list of directory (coresponding to first member)
        for idir in range(len(dirlist)):
            direcfc0=dirlist[idir]
            date=sdates[idir]
            #print("idir")
            #print(idir)
            filememblst=[]
            for imemb in memblst:
                membname="fc%i"%imemb
                direc=direcfc0.replace("fc0",membname)
                print(direc)
                #select if SH or GG files
                if (varname=="tas"):
                     vartype="GG"
                elif ("ta" in varname) | ("ua" in varname) | ("va" in varname) | ("ga" in varname):
                     vartype="SH"
                else:
                     vartype="GG"
                #find the grib files
                filelistmon=glob(direc+'/*'+vartype+"*.grb")
                #order the list
                filelistmon.sort()
                #count the nb of file to know the nb of leadtime
                nbfor=len(foretimelst)
                formonlst=list(foretimelst)
                #if the list of forecast time to consider is empty take all the months available
                if (nbfor==0):
                    nbfor=len(filelistmon)
                    formonlst=range(nbfor)
                filelistmonout=[]
                #concatenate the leadtimes
                for ifile in formonlst:
                    #fileout=filelistmon[ifile].split("/")[-1]
                    #fileout=fileout.replace(".grb",".nc")
                    #include sellevl here, when needed
            
                    fileout=cdo.selvar(oldname,
                                 input=filelistmon[ifile], 
                                   options="-R -r -t ecmwf -f nc")
                    filelistmonout.append(fileout)
                filemembout=cdo.mergetime(input=filelistmonout, 
                                   options= "-O -r -f nc")
                 #print(filelistmonout)                  
                filememblst.append(filemembout)

            fileout=varname + "_" + str(date).replace("-","")+".nc"
            print(imemb)
            cdo.chvar(oldname,varname, 
                      input=cdo.merge(input=filememblst,
                                      options="-O -r"),
                      output=fileout, options="-r", 
                      force =  False)
            
            filelist.append(fileout)
        #print(filelist)
        self.set_filelist(filelist)
        self.set_freq(get_freq(filelist[0]))
        self.set_nmember(nmember)
        self.see()
         
    def ready(self, date1, date2, memblst=[], foretimelst=[]):
        """ make the Data object ready to calculate statistic
        """
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        dataname=self.get_dataname()
        varname=self.get_varname()
        self.set_inputdir("./")
        self.set_outputdirAuto() #look for the output directory
        test=self.set_files(date1,date2)
        self.see()
        if test == 1:
            if varname == "tasmin":
                self.set_varname("tas")
                self.set_inputdirAuto()
                self.set_outputdirAuto() #look for the output directory
                self.set_files(date1,date2) #look for the files between date1 and date2 
                self.format_files_nc(memblst) #transform grib in nc files cdo compatible
                self.cdo_statistic("daymin","tasmin")
            elif varname == "tasmax":
                self.set_varname("tas")
                self.set_inputdirAuto()
                self.set_outputdirAuto() #look for the output directory
                self.set_files(date1,date2) #look for the files between date1 and date2 
                self.format_files_nc(memblst) #transform grib in nc files cdo compatible
                self.cdo_statistic("daymax","tasmax")
            elif varname == "prlr":
                auxlsp=DataEC3(dataname,"lsp")
                auxlsp.set_inputdir("./")
                test=auxlsp.set_files(date1,date2)
                if test == 1:
                    auxlsp.set_inputdirAuto()
                    #look for the files between date1 and date2 
                    auxlsp.set_files(date1,date2)
                    #transform grib in nc files cdo compatible
                    auxlsp.format_files_nc(memblst) 

                    auxcp=DataEC3(dataname,"cp")
                    auxcp.set_inputdir("./")
                    test=auxcp.set_files(date1,date2)
                    if test == 1:
                        auxcp.set_inputdirAuto()
                         #look for the files between date1 and date2 
                        auxcp.set_files(date1,date2)
                        #transform grib in nc files cdo compatible
                        auxcp.format_files_nc(memblst) 

                    aux=weighted_sum((auxlsp, auxcp), (1,1), varname)
                    self.copy(aux)
                    self.cdo_statistic("daysum","prlr")
                self.set_outputdir("/esnas/exp/ecearth/"+dataname+"/monthly_statistics"+"") #look for the output directory
            else:
                self.set_varname(varname)
                self.set_inputdirAuto()
                self.set_outputdirAuto() #look for the output directory
                self.set_files(date1,date2) #look for the files between date1 and date2 
                self.format_files_nc(memblst) #transform grib in nc files cdo compatible
            self.see()
               
