import datetime
import time
from data_mod import DataMod
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
from cdo import *
import copy
from scipy.io.netcdf import *
import numpy
import shutil
from common import *



class DataEns(DataMod):
    """ classe of data for the ENSEMBLES multi-model
    """
    def __init__(self, dataname, varname):
        DataMod.__init__(self, dataname, varname)
        self._nmenber=9
        self._freq=datetime.timedelta(0, 0, 0, 0, 0, 24)

    

    def set_inputdirAuto(self):
        """defining the path to look for the model daily data"""
        dataexp = { "EnsCmccSeas":"/cfunas/exp/ENSEMBLES/seasonal/cmcc/daily_mean/" ,
                    "EnsIfmSeas":"/cfunas/exp/ENSEMBLES/seasonal/ifm/daily_mean/",
                    "EnsMetfrSeas":"/cfunas/exp/ENSEMBLES/seasonal/metfr/daily_mean/",
                    "EnsUkmoSeas":"/cfunas/exp/ENSEMBLES/seasonal/ukmo/daily_mean/" , 
                    "EnsEcmwfSeas":"/cfunas/exp/ENSEMBLES/seasonal/ecmwf/daily_mean/"
                    } 
        #  # here add the general case when needed (m02j, m02m....)
        dataname=self.get_dataname()
        varname=self.get_varname()
        try:
            inputdir=dataexp.get(dataname) +  varname + "/"
        except:
            print "The experience " + dataname + " is not yet available in the function"
            sys.exit(0)
        
        if (not os.path.isdir(inputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)

        self.set_inputdir(inputdir)



    def format_files_nc(self):
        """this fonction check if the file in the filelist 
        have a proper format.
        get the variables leadtime and freq
        the calendar is checked and set if needed
        the variables incompatibles with cdo are removed"""

        cdo=Cdo()

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()

        #caculate the frequency and check in all files 
        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]

            #test if the taxis axis is well defined
            f=settaxis(dataname, varname, f, d, freq)
            #add here solution for cdo imcompatibilities
            

            filelist[i]=f

        self.set_freq(freq)
        self.set_filetype(freq)


            

    def ready(self, date1, date2):
        """ Make the Data object ready to calculate statistics """
        self.set_inputdirAuto() #look for the input directory
        self.set_outputdirAuto() #look for the output directory
        self.set_files(date1,date2) #look for the files between date1 and date2 
        self.format_files_nc() #set file calendar
        self.see()
