import datetime
import time
from data_mod import DataMod
import sys
import os
from glob import glob
import sets
from dateutil.relativedelta import *
import cdo as cdop
import copy
from scipy.io.netcdf import *
import numpy
import shutil
from common import *
import tarfile as tar
import gzip
from netCDF4 import Dataset as nc
#from nco import Nco



class DataGridNemo(DataMod):
    """ classe of data for all experiments realized 
    with autosubmit with the standard outclass 
    """
    def __init__(self, dataname, varname):
        DataMod.__init__(self, dataname, varname)
        self.set_freq(datetime.timedelta(0, 0, 0, 0, 0, 6))
        self._maskfile="/esnas/autosubmit/con_files/mesh_mask_nemo.Ec3.1_O1L46.nc"

    def set_inputdirAuto(self):
        """defining the path to look for the model daily data"""
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        inputdir = os.path.join("/esnas/exp/ecearth/",
                                dataname)
        self.set_inputdir(inputdir)

    def set_outputdirAuto(self):
        """defining the path to store the output"""
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        outputdir = "/esnas/exp/ecearth/"+dataname+"/"+""
        if (not os.path.isdir(outputdir)):
            print "This directory does not exist:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)

    def set_files(self, date1, date2, memblst=[]):
        """set the list of stardates and correponding directories
        as for these type of experiments the members and leadtime
        are stored in different file we first find the correct 
        path to the directories"""

        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]
        filetype="dir"
        if (inputdir=="./"):
            filetype="nc"
            filebase=self.get_varname() + "_" + date1.strftime("%Y%m%d")+".nc"
            filename=glob(inputdir+filebase)
            if len(filename)== 0:
                print("no nc file in "+inputdir)
                return 1
            filelist.append(filename[0])
            datelist.append(date1)
            date1 = date1 + relativedelta(years=+1)
            d=date1
            #loop on the date to find all the files
            while d <= date2:
                filebase=self.get_varname() + "_" +d.strftime("%Y%m%d") +".nc"
                filename=glob(os.path.join(inputdir,filebase))
                if len(filename)== 0:
                    print "the file %s does not exit " % inputdir+filebase+"."+filetype
                    print "while the file %s exists "  % inputdir+filelist[0]
                    print "all file must have the same type (grb or nc)"
                    return 1
                datelist.append(d)
                filelist.append(filename[0])
                d = d + relativedelta(years=+1)
        #Otherwise look for the file in the storage
        else:
            print("no file in the working directory: go in the storage")
            if (not os.path.isdir(inputdir)):
                print "This directory does not exist:"
                print inputdir
                return 1

            filebase=os.path.join(inputdir,
                                  date1.strftime("%Y%m%d"),"fc0","outputs")
            filelist.append(filebase)
            datelist.append(date1)
            d=date1 + relativedelta(years=+1)
            #loop on the date to find all the files
            while d <= date2:
                filebase=os.path.join(inputdir,
                                  d.strftime("%Y%m%d"),"fc0","outputs")
                datelist.append(d)
                filelist.append(filebase)
                d = d + relativedelta(years=+1)
        self.set_filetype(filetype)
        self.set_filelist(filelist)
        self.set_sdates(datelist)
    
    def format_files_nc(self, memblst=[], regname="", fldmean=False):
        """Open the tar file and extract the variables of interest
        into the grid_T, U, V file
        memblst: list of member to extract, if empty all are used
        regname: name of a region to be selected, if empty no region selected
        fldmean: to average the field into a given region, if empty no average
        """
        #needed to use cdo
        cdo=cdop.Cdo()
        #make sure that the tmp dir is not /tmp
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        inputdir=self.get_inputdir()
        dirlist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        nmember=self.get_nmember()
        filetype=self.get_filetype()
        maskfile=self._maskfile
        filelist=[]
        #test
        region, notuseful=regcoord(regname)
        print("I am here2")
       
        #define the grid type and mask variable according to the variable
        if varname in ["sozotaux"]:
            grid="U"
            varmask="umask"
        elif varname in ["sometauy"]:
            grid="V"
            varmask="vmask"
        else:
            grid="T"
            varmask="tmask"
        #get the number of members, which can be define as an argument
        #otherwise all the members are used
        nmember=len(memblst)
        if len(memblst)==0:
            lstdir=glob(os.path.join(inputdir,
                                     (sdates[0].strftime("%Y%m%d")+"/*")))
            lstdir.sort()
            memblst=[int(x.split("/fc")[-1]) for x in lstdir]
            nmember=len(memblst)
        
        #loop over the list of directory (coresponding to first member)
        for idir in range(len(dirlist)):
            print(sdates[idir])
            #get the directory of the first member
            direcfc0=dirlist[idir]
            filetarfc0=glob(direcfc0+'/*'+"MMO_"+dataname+"_"+sdates[idir].strftime("%Y%m%d")+"*.tar")[0]
            
            filememblst=[]
            print(memblst)
            for imemb in memblst:
                print(imemb)
                #find the tar files corresponding to the considered member
                filetar=filetarfc0.replace("fc0","fc%i"%imemb)
                #untar the files
                t = tar.open(filetar, 'r')
                files=t.getnames()
                #find the files corresponding to 1d grid_T in the gzip
                filegz=""
                for f in files:
                    #find the grib file we are looking at
                    if ("1d" in f) and ("grid_"+grid in f):
                        filegz = f
                if filegz == "":
                    print("no daily grid_"+grid+" file in"+tarfile)
                    return 1
                #gunzip filein
                filein = filegz.replace(".gz","").replace("_1d","_fc%i_1d"%imemb)
                #check if filein already in the workdir
                #if not unzip it
                if len(glob(filein))==0:
                    #first extract from the tar file
                    t.extract(filegz, path=".")
                    inF = gzip.open(filegz, 'rb')
                    outF = open(filein, 'wb')
                    outF.write( inF.read() )
                    inF.close()
                    outF.close()
                    #remove the .gz
                    os.remove(filegz)
                t.close()
                #define the cdo comand to select a region
                selreg=""
                #if the region option is activated
                if (regname)!="":
                    #find the longitude and latitude according to the region name
                    region, notuseful=regcoord(regname)
                    
                    fmin=nc(maskfile, "r")
                    #read the longitude
                    lon=np.array(fmin.variables["nav_lon"][:].squeeze())
                    #read the latitude
                    lat=np.array(fmin.variables["nav_lat"][:].squeeze())
                    fmin.close()

                    lonmin, lonmax, latmin, latmax = region
                    #to select a region, find first the index coresponding
                    box=np.where((lon>lonmin)&(lon<lonmax)&(lat>latmin)&(lat<latmax))
                    lonbox1,lonbox2=box[1].min(),box[1].max()
                    latbox1,latbox2=box[0].min(),box[0].max()
                
                    #cdo command to select a region
                    selreg="-selindexbox,%i,%i,%i,%i "%(lonbox1,lonbox2,latbox1,latbox2)

                #define the cdo comand the fieldmean
                areaav=""
                regnamenew=regname
                if fldmean:
                    #construct the weight
                    weights=cdo.copy(input=selreg+"-mul -selvar,e1t "+ maskfile+ " -selvar,e2t "+ maskfile)
                    #cdo command to apply to area average (with weights)
                    areaav="-fldmean -setgridarea,"+weights
                    regnamenew=regname+"mean"

                #define the mask
                mask = cdo.copy(input=selreg+"-sellevidx,1 -selvar,"+varmask+" "+" "+maskfile)

                
                #select the variable, the region  and average it of the option is activated
                cdocomand=areaav+" -div "+selreg+" -selvar,"+varname+" "+filein+" "+mask
                #create the file
                filememb=cdo.copy(input=cdocomand)
                filememblst.append(filememb)
                #name of the output file
                fileout=regnamenew+"_"+varname+"_"+sdates[idir].strftime("%Y%m%d")+".nc"
            #concatenate the members
            if len(glob(fileout))!=0:
                os.remove(fileout)
            fileout=cdo.merge(input=" ".join(filememblst), output=fileout, option="-O -f nc")

            filelist.append(fileout)
    
        self.set_filelist(filelist)
        self.set_freq(get_freq(filelist[0]))
        self.set_nmember(nmember)
        self.see()
